@@ -1,11 +1,13 @@
 F.cjs.estim <- function(capture, survival, histories, cap.init, sur.init, 
-    group, algorithm=1, cov.meth=1, nhat.v.meth=1, 
-    c.hat=-1.0, df=NA, intervals=rep(1,ncol(histories)-1), conf=0.95){
+    group, nhat.v.meth=1, 
+    c.hat=-1.0, df=NA, intervals=rep(1,ncol(histories)-1), conf=0.95, 
+    control=mra.control() ){
 
 start.sec <- proc.time()[3]
 
 run.date <- Sys.time()
 
+# ---- Verify some of the inputs
 if( missing(histories) ){
     stop("Capture histories matrix must be specified")
 }
@@ -18,11 +20,13 @@ if( missing(survival) ){
 
 if( length(union( unique(histories), c(0,1,2))) > 3 ) stop("Capture histories must consist of 0's, 1's, and 2's only.")
 
-if( !any( algorithm == c(1,2,3) ) ){
-    warning(paste("Algorithm", algorithm, "not recognized. Algorithm 1 used"))
-    algorithm <- 1
+if( !is.numeric(nhat.v.meth) || !(nhat.v.meth %in% c(1,2,3)) ){
+ 	stop("value of 'nhat.v.meth' must be 1, 2, or 3")
 }
 
+# Validity of cov.meth and algorithm and other inputs is determined in mra.control()
+
+# ---- Initialize some variables
 hist.name <- deparse(substitute(histories))
 cr.call <- match.call()
 
@@ -57,6 +61,13 @@ if( missing(sur.init) ){
     sur.init <- c(sur.init, rep(0, ny-length(sur.init)))
 } 
 
+#   Set up the tolerance vector, if not specified, or if not long enough
+if( length(control$tol) < (nx+ny) ){
+    control$tol <- rep(control$tol, trunc((nx+ny) / length(control$tol))+1)[1:(nx+ny)]
+} else if( length(control$tol > (nx+ny)) ){
+    control$tol <- control$tol[1:(nx+ny)]
+}
+
 
 #   Fix up the group variable
 if( missing( group )){
@@ -66,16 +77,6 @@ if( missing( group )){
     ng <- length(unique(group))
 }
 
-
-
-#   Determine covariance method.  1 = numeric 2nd derivatives, 2 = inverse of optimization Hessian
-if( !any( cov.meth == c(1,2) ) ){
-    cat("Covariance method must be either 1 = numeric 2nd derivatives (default), or 2 = Hessian of optimazation\n")
-    cat("Using 2nd derivative method (cov.meth = 1).\n")
-    cov.meth <- 1 
-}
-
-
 #   Transfer over c.hat
 vif <- c.hat
 
@@ -86,145 +87,136 @@ covariance <- matrix( 0, nx+ny, nx+ny )
 p.hat <- se.p.hat <- s.hat <- se.s.hat <- matrix( 0, nan, ns )
 exit.code <- cov.code <- 0
 n.hat <- se.n.hat <- rep(0, ns)
+# on entry to .Fortran, maxfn is maximum number of function evals.  On return, maxfn is actual number of evaluations.
+maxfn <- control$maxfn  
 if( is.na(df) ){
     df.estimated <- 1  # Have MRAWIN estimate rank of var-covar matrix
 } else {
     df.estimated <- 0  # Don't bother, df either set by user or will use nx+ny
 }
 
-cat( "Calling MRA DLL to maximize likelihood.  Please wait...\n")
+if(control$trace) cat( "Calling MRA DLL to maximize likelihood.  Please wait...\n")
 
 ans <- .Fortran( "cjsmod", 
-        as.integer(nan), 
-        as.integer(ns), 
-        as.integer(nx), 
-        as.integer(ny), 
-        as.integer(ng), 
-        as.integer(histories), 
-        as.integer(group), 
-        as.integer(algorithm), 
-        as.integer(cov.meth), 
-        as.integer(nhat.v.meth), 
-        as.double(covars$capX), 
-        as.double(covars$survX), 
-        as.double(cap.init), 
-        as.double(sur.init), 
-        as.double(loglik), 
-        as.double(deviance), 
-        as.double(aic), 
-        as.double(qaic), 
-        as.double(vif), 
-        as.double(chisq.vif), 
-        as.double(df.vif), 
-        as.double(parameters),
-        as.double(se.param), 
-        as.double(covariance), 
-        as.double(p.hat), 
-        as.double(se.p.hat), 
-        as.double(s.hat), 
-        as.double(se.s.hat), 
-        as.double(n.hat), 
-        as.double(se.n.hat), 
-        as.integer(exit.code), 
-        as.integer(cov.code), 
-        as.integer(df.estimated), 
-        as.double(intervals), 
+        nan         = as.integer(nan), 
+        ns          = as.integer(ns), 
+        nx          = as.integer(nx), 
+        ny          = as.integer(ny), 
+        ng          = as.integer(ng), 
+        histories   = as.integer(histories), 
+        group       = as.integer(group), 
+        algorithm   = as.integer(control$algorithm), 
+        cov.meth    = as.integer(control$cov.meth), 
+        trace       = as.integer(control$trace),
+        nhat.v.meth = as.integer(nhat.v.meth), 
+        capX        = as.double(covars$capX), 
+        survX       = as.double(covars$survX), 
+        cap.init    = as.double(cap.init), 
+        sur.init    = as.double(sur.init), 
+        maxfn       = as.integer(maxfn),
+        beta.tol.vec= as.double(control$tol), 
+        loglik      = as.double(loglik), 
+        deviance    = as.double(deviance), 
+        aic         = as.double(aic), 
+        qaic        = as.double(qaic), 
+        vif         = as.double(vif), 
+        chisq.vif   = as.double(chisq.vif), 
+        df.vif      = as.double(df.vif), 
+        parameters  = as.double(parameters),
+        se.parm     = as.double(se.param), 
+        covariance  = as.double(covariance), 
+        p.hat       = as.double(p.hat), 
+        se.p.hat    = as.double(se.p.hat), 
+        s.hat       = as.double(s.hat), 
+        se.s.hat    = as.double(se.s.hat), 
+        n.hat       = as.double(n.hat), 
+        se.n.hat    = as.double(se.n.hat), 
+        exit.code   = as.integer(exit.code), 
+        cov.code    = as.integer(cov.code), 
+        df.estimated= as.integer(df.estimated), 
+        intervals   = as.double(intervals), 
         PACKAGE="mra" 
         ) 
 
 #   Remember to add PACKAGE="mra" back into the .Fortran call above.
 
-cat(paste("Returned from MRAWIN. Details in MRA.LOG.\n", sep=""))
+if( control$trace ) cat(paste("Returned from MRAWIN. Fitting details written to MRA.LOG.\n", sep=""))
 
-loglik     <- ans[[15]]
-deviance   <- ans[[16]] 
-aic        <- ans[[17]] 
-qaic       <- ans[[18]] 
-vif        <- ans[[19]] 
-chisq.vif  <- ans[[20]] 
-df.vif     <- ans[[21]] 
-parameters <- ans[[22]]
-se.param   <- ans[[23]] 
-covariance <- ans[[24]] 
-p.hat      <- ans[[25]] 
-se.p.hat   <- ans[[26]] 
-s.hat      <- ans[[27]] 
-se.s.hat   <- ans[[28]]
-n.hat      <- ans[[29]]
-se.n.hat   <- ans[[30]]
-exit.code  <- ans[[31]]
-cov.code   <- ans[[32]]
-df.estimated <- ans[[33]]
 
-# ----- Reset missing standard errors to NA
-se.param[ se.param < 0 ] <- NA
+## ----- Reset missing standard errors to NA
+ans$se.param[ ans$se.param < 0 ] <- NA
 
 # ----- R does not preserve the matrix structures in .Fortran call.  Put matricies, 
 #   which are now vectors, back to matricies.
-covariance <- matrix( covariance, nrow=nx+ny ) 
-p.hat      <- matrix( p.hat, nrow=nan )
-se.p.hat   <- matrix( se.p.hat, nrow=nan )
-s.hat      <- matrix( s.hat, nrow=nan )
-se.s.hat   <- matrix( se.s.hat, nrow=nan )
-
-
+ans$covariance <- matrix( ans$covariance, nrow=nx+ny ) 
+ans$p.hat      <- matrix( ans$p.hat, nrow=nan )
+ans$se.p.hat   <- matrix( ans$se.p.hat, nrow=nan )
+ans$s.hat      <- matrix( ans$s.hat, nrow=nan )
+ans$se.s.hat   <- matrix( ans$se.s.hat, nrow=nan )
 
 # ----- Work out exit codes
-if( exit.code==0 ){
+if( ans$exit.code==0 ){
     exit.mess = "FAILURE: Initial Hessian not positive definite"
-} else if( exit.code == 1 ){
+} else if( ans$exit.code == 1 ){
     exit.mess = "SUCCESS: Convergence criterion met"
-} else if( exit.code == 2 ){
+} else if( ans$exit.code == 2 ){
     exit.mess = "FAILURE: G'dX > 0, rounding error"
-} else if( exit.code == 3 ){
+} else if( ans$exit.code == 3 ){
     exit.mess = "FAILURE: Likelihood evaluated too many times"
-} else if( exit.code == -1 ){
+} else if( ans$exit.code == -1 ){
     exit.mess = "FAILURE: Algorithm 2 not implimented yet.  Contact Trent McDonald."
 } else {
     exit.mess = "Unknown exit code"
 }
 
-if(algorithm == 1){
-    message <- "Optimization by VA09AD."
+if(control$algorithm == 1){
+    alg.mess <- "Optimization by VA09AD."
 } else {
-    message <- "Unknown optimization routine."
+    alg.mess <- "Unknown optimization routine."
 }
 
-if(cov.meth == 1){
+if(control$cov.meth == 1){
     cov.mess = "Covariance from numeric derivatives."
-} else if (cov.meth == 2){
+} else if (control$cov.meth == 2){
     cov.mess = "Covariance from optimization Hessian."
 } else {
     cov.mess = "Unkown covariance method."
 }
 
-if(cov.code == 0){
+if(ans$cov.code == 0){
     cov.mess = paste( cov.mess,  "SUCCESS: Non-singular covariance matrix.")
-} else if( cov.code == 1) {
+} else if( ans$cov.code == 1) {
     cov.mess = paste( cov.mess,  "ERROR: COVARIANCE MATRIX IS SINGULAR.")
 } else {
     cov.mess = paste( cov.mess,  "ERROR: NON-NEGATIVE DEFINITE COVARIANCE MATRIX.")
 }
 
-# Remove trailing blanks from message
-message <- c( paste( message, exit.mess),
-        cov.mess)
-cat(paste( "\t(", message, ")\n", sep="" ))
-
+# do some printouts
+if( ans$exit.code != 1 ){
+	cat(exit.mess)
+	cat("\n")
+} else if( ans$cov.code != 0 ){
+	cat(cov.mess)
+	cat("\n")
+} else if( control$trace ){
+	cat(paste( alg.mess, "\n", sep="" ))
+	cat(paste( exit.mess, "\n", sep="" ))
+	cat(paste( cov.mess, "\n", sep="" ))
+}
 
 # ----- Fix up capture and survival estimates
 #   Wipe out the first capture probability and the last survival probability. These are computed 
 #   by mrawin5 if there are covariates out there, but they are not used in the likelihood, and 
 #   we can't really estimate them.
-p.hat[,1] <- NA
-s.hat[,ns] <- NA
-se.s.hat[,ns] <- NA
-se.p.hat[,1] <- NA
+ans$p.hat[,1] <- NA
+ans$s.hat[,ns] <- NA
+ans$se.s.hat[,ns] <- NA
+ans$se.p.hat[,1] <- NA
 
-capcoef <- parameters[1:nx]
-se.capcoef <- se.param[1:nx]
-surcoef <- parameters[(nx+1):(nx+ny)]
-se.surcoef <- se.param[(nx+1):(nx+ny)]
+capcoef <- ans$parameters[1:nx]
+se.capcoef <- ans$se.param[1:nx]
+surcoef <- ans$parameters[(nx+1):(nx+ny)]
+se.surcoef <- ans$se.param[(nx+1):(nx+ny)]
 
 names(capcoef) <- covars$cap.vars
 names(se.capcoef) <- covars$cap.vars
@@ -237,14 +229,14 @@ dimnames(covariance) <- list( c( paste("cap.",names( capcoef ),sep=""), paste("s
 
 # ----- Fix up number of parameters.
 if( is.na(df) ){
-    df <- df.estimated   # use the rank estimated by MRAWIN
+    df <- ans$df.estimated   # use the rank estimated by MRAWIN
 } else if( df <= 0 ){
     df <- nx+ny  # assume full rank
 } # else use the values supplied by user (unchanged from input)
 
 # ----- Now that df is computed, recompute fit statistics
-aic <- -2*loglik + 2*df
-qaic <- -2*(loglik/vif) + 2*df
+aic <- -2*ans$loglik + 2*df
+qaic <- -2*(ans$loglik/ans$vif) + 2*df
 aicc <- aic + (2*df*(df+1))/(nan - df - 1)
 qaicc <- qaic + (2*df*(df+1))/(nan - df - 1)
 
@@ -272,55 +264,59 @@ aux <- list( call=cr.call,
 
 
 # ----- Fix up the estimates of N.
-names(n.hat) <- dimnames(histories)[[2]]
+names(ans$n.hat) <- dimnames(histories)[[2]]
 num.observed <- c( t( rep(1, nrow(histories)) ) %*% (histories >= 1) )
 crit.value <- qnorm( 1-((1-conf)/2) )
-lower.ci <- n.hat - crit.value * se.n.hat
+lower.ci <- ans$n.hat - crit.value * ans$se.n.hat
 lower.ci <- ifelse(lower.ci < num.observed, num.observed, lower.ci)
-upper.ci <- n.hat + crit.value * se.n.hat
+upper.ci <- ans$n.hat + crit.value * ans$se.n.hat
 
-n.hat[1] <- NA  # can't estimate the first N
-se.n.hat[1] <- NA
+ans$n.hat[1] <- NA  # can't estimate the first N
+ans$se.n.hat[1] <- NA
 lower.ci[1] <- NA
 upper.ci[1] <- NA
 
+# ----- Compute execution time here so can store in output.
+ex.time <- (proc.time()[3] - start.sec) / 60
 
 # ----- Done. Put into a 'CR' object.
 
 ans <- list( histories=histories, 
     aux=aux, 
     intervals=intervals[1:(ns-1)], 
-    loglike=loglik, 
-    deviance=deviance, 
+    loglike=ans$loglik, 
+    deviance=ans$deviance, 
     aic=aic, 
     qaic=qaic, 
     aicc=aicc, 
     qaicc=qaicc,
-    vif=vif, 
-    chisq.vif=chisq.vif, 
-    vif.df=df.vif, 
-    parameters=parameters, 
-    se.param=se.param, 
+    vif=ans$vif, 
+    chisq.vif=ans$chisq.vif, 
+    vif.df=ans$df.vif, 
+    parameters=ans$parameters, 
+    se.param=ans$se.param, 
     capcoef=capcoef, 
     se.capcoef=se.capcoef, 
     surcoef=surcoef, 
     se.surcoef=se.surcoef, 
-    covariance=covariance,
-    p.hat=p.hat, 
-    se.p.hat=se.p.hat, 
-    s.hat=s.hat, 
-    se.s.hat=se.s.hat, 
+    covariance=ans$covariance,
+    p.hat=ans$p.hat, 
+    se.p.hat=ans$se.p.hat, 
+    s.hat=ans$s.hat, 
+    se.s.hat=ans$se.s.hat, 
     df=df, 
-    message=message, 
-    exit.code=exit.code, 
-    cov.code=cov.code, 
-    cov.meth=cov.meth, 
-    n.hat = n.hat, 
-    se.n.hat=se.n.hat, 
+    control=control,
+    message=paste(c(alg.mess,exit.mess,cov.mess),"\n"), 
+    exit.code=ans$exit.code, 
+    cov.code=ans$cov.code, 
+    fn.evals=ans$maxfn,
+    ex.time = ex.time,
+    n.hat =ans$n.hat, 
+    se.n.hat=ans$se.n.hat, 
     n.hat.lower = lower.ci, 
     n.hat.upper = ceiling(upper.ci),
     n.hat.conf = conf, 
-    nhat.v.meth = nhat.v.meth, 
+    nhat.v.meth = ans$nhat.v.meth, 
     num.caught=num.observed)
     
 class(ans) <- c("cjs", "cr")
@@ -333,11 +329,12 @@ ans$residuals <- residuals( ans, type="pearson" )
 ans$resid.type <- "pearson"
 
 
-ex.time <- (proc.time()[3] - start.sec) / 60
-if( ex.time < 0.01666667 ){
-    cat("\t(Execution time < 1 second)\n")
-} else {
-    cat(paste("\t(Execution time =", round(ex.time,2), "minutes)\n"))
+if( control$trace ){
+    if( ex.time < 0.01666667 ){
+        cat("\t(Execution time < 1 second)\n")
+    } else {
+        cat(paste("\t(Execution time =", round(ex.time,2), "minutes)\n"))
+    }
 }
 
 
